<?php

namespace App\Http\Controllers;

use App\Models\Mhcbinding;
use App\Models\Omics;
use App\Exports\MhcbindingExportClass;
use App\Exports\MhcbindingExportDetailClass;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Maatwebsite\Excel\Facades\Excel;

class MhcbindingController extends Controller
{
    //
    public function index(Request $request)
    {

//        DB::enableQueryLog();
        $perPage = $request->input('pageSize', 10);
        $sql = Mhcbinding::join("omics", "omics.Sample ID", "=", "mhcbinding.Sample ID")
            ->join("blast", "blast.Reads ID", "=", "mhcbinding.Reads ID")
            ->when($request->has('keywords'), function($query) use ($request) {
            $query->where("mhcbinding.Reads ID", "like", "%" . $request->get('keywords') . "%");
        })
            ->when($request->has('name'), function($query) use ($request) {
            $query->where("mhcbinding.Reads ID", "=", $request->get('name'));
        })
            ->when($request->has('Patient_ID'), function($query) use ($request) {
                $query->where("Patient ID", "=", $request->get('Patient_ID'));
            })
            ->when($request->has('Stitle'), function($query) use ($request) {
                $query->where("Stitle", "=", $request->get('Stitle'));
            })
            ->when($request->has('SourceTissue'), function($query) use ($request) {
                $query->where("omics.Source tissue", "=", $request->get('SourceTissue'));
            })
            ->when($request->has('HLA_Allele'), function($query) use ($request) {
                $query->where("mhcbinding.HLA Allele", "=", $request->get('HLA_Allele'));
            })
            ->when($request->has('Best_IC50_Score'), function($query) use ($request) {
                $query->where("Best IC50 Score", "=", $request->get('Best_IC50_Score'));
            })
            ->when($request->has('Best_Percentile'), function($query) use ($request) {
                $query->where("Best Percentile", "=", $request->get('Best_Percentile'));
            })


            ->when($request->has('Epitope'), function($query) use ($request) {
            $query->where("Epitope Seq", "=", $request->get('Epitope'));
        })
            ->when($request->has('gene'), function($query) use ($request) {
            $query->where("blast.Stitle", "=", $request->get('gene'));
        })->when($request->has('prop'), function($query) use ($request) {
            $prop = $request->get('prop');
            switch ($prop) {
                case 'Sample ID': $prop = "omics.Sample ID"; break;
            }
            if ($request->get("order")=="descending"){
                $query->orderBy($prop, "desc");
            }else{
                $query->orderBy($prop, "asc");
            }
        });
        if($request->get('download')==1){
            $data = $sql->get();
            return Excel::download(new MhcbindingExportClass($data), 'data.xlsx');
        }else if($request->get('download')==2){
            $data = $sql->get();
            return Excel::download(new MhcbindingExportDetailClass($data), 'data.xlsx');
        }else{
            return response()->json($sql->paginate($perPage));
        }

    }

      /**
     * 统计HLA Allele
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function GroupByHLA(Request $request): \Illuminate\Http\JsonResponse
    {
        $res =  Mhcbinding::select( DB::raw("COUNT(DISTINCT `HLA Allele`) as num"))
            ->get();
        return response()->json($res);
    }

     /**
     * 统计HLA Allele
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function GroupByCategory(Request $request): \Illuminate\Http\JsonResponse
    {
        $res =  Mhcbinding::select( DB::raw("count(DISTINCT SUBSTRING_INDEX(SUBSTRING_INDEX( `Reads ID`,'|', 2 ),'|',-1)) as num"))
            ->get();
        return response()->json($res);
    }


     /**
     * 统计HLA Allele
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function GroupByBelong(Request $request): \Illuminate\Http\JsonResponse
    {
        $res =  Mhcbinding::select( DB::raw("count(DISTINCT SUBSTRING_INDEX(SUBSTRING_INDEX(SUBSTRING_INDEX( `Reads ID`,'|', 2 ),'|',-1),'.',1)) as num"))
            ->get();
        return response()->json($res);
    }


    /**
     * 饼图2
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function GroupBySource(Request $request): \Illuminate\Http\JsonResponse
    {
//        DB::enableQueryLog();
        $group = Mhcbinding::groupBy("HLA Allele")->select("HLA Allele", DB::raw("count(`Sample ID`) as num"))->get();
//        dd(DB::getQueryLog());
        return response()->json($group);
    }

    /**
     * 柱状图1
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function GroupByType(Request $request): \Illuminate\Http\JsonResponse
    {
//        DB::enableQueryLog();
        $res =  Mhcbinding::join("omics", "omics.Sample ID", "=", "mhcbinding.Sample ID")
            ->selectRaw("omics.`Source tissue`, SUBSTRING(`HLA Allele`, 1, 3) AS type, count(*) as num")
            ->groupBy("Source tissue", "type")->orderBy("Source tissue")
            ->get();
//        dd(DB::getQueryLog());
        $data = [];
        foreach ($res as $key => $value) {
            if (!data_get($data,$value['Source tissue'])){
                $data[$value['Source tissue']] = ['I'=>0,'II'=>0];
            }
            if ($value['type'] == 'HLA'){
                $data[$value['Source tissue']]['I'] += $value['num'];
            }else{
                $data[$value['Source tissue']]['II'] += $value['num'];
            }
        }
        return response()->json($data);
    }

     /**
     * 柱状图2
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function GroupByReadsID(Request $request): \Illuminate\Http\JsonResponse
    {
//        DB::enableQueryLog();
        $res =  Mhcbinding::join("omics", "omics.Sample ID", "=", "mhcbinding.Sample ID")
            ->selectRaw("omics.`Source tissue`, SUBSTRING(`HLA Allele`, 1, 3) AS type, count(DISTINCT SUBSTRING_INDEX(SUBSTRING_INDEX( `Reads ID`,'|', 2 ),'|',-1)) as num")
            ->groupBy("Source tissue", "type")->orderBy("Source tissue")
            ->get();
//        dd(DB::getQueryLog());
//        dd($res->toArray());
        $data = [];
        foreach ($res as $key => $value) {
            if (!data_get($data,$value['Source tissue'])){
                $data[$value['Source tissue']] = ['I'=>0,'II'=>0];
            }
            if ($value['type'] == 'HLA'){
                $data[$value['Source tissue']]['I'] += $value['num'];
            }else{
                $data[$value['Source tissue']]['II'] += $value['num'];
            }
        }
        return response()->json($data);
    }


     /**
     * 柱状图3
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function GroupByReadsID3(Request $request): \Illuminate\Http\JsonResponse
    {
//                DB::enableQueryLog();
        $res =  Mhcbinding::join("omics", "omics.Sample ID", "=", "mhcbinding.Sample ID")
            ->selectRaw("omics.`Source tissue`, SUBSTRING(`HLA Allele`, 1, 3) AS type, count(DISTINCT SUBSTRING_INDEX(SUBSTRING_INDEX(SUBSTRING_INDEX( `Reads ID`,'|', 2 ),'|',-1),'_',1)) as num")
            ->groupBy("Source tissue", "type")->orderBy("Source tissue")
            ->get();
//                dd(DB::getQueryLog());
//        dd($res->toArray());
        $data = [];
        foreach ($res as $key => $value) {
            if (!data_get($data,$value['Source tissue'])){
                $data[$value['Source tissue']] = ['I'=>0,'II'=>0];
            }
            if ($value['type'] == 'HLA'){
                $data[$value['Source tissue']]['I'] += $value['num'];
            }else{
                $data[$value['Source tissue']]['II'] += $value['num'];
            }
        }
        return response()->json($data);
    }

    public function readsIdList(Request $request): \Illuminate\Http\JsonResponse
    {
        $res = Mhcbinding::when($request->get("keywords"), function ($query) use ($request) {
            $query->where("Reads ID", "like", "%{$request->get('keywords')}%");
        })->selectRaw(DB::raw("SUBSTRING_INDEX(SUBSTRING_INDEX(SUBSTRING_INDEX( `Reads ID`,'|', 2 ),'|',-1),'.',1) as label"))->limit(100)->get();
        return response()->json($res);
    }
}

