<?php

namespace App\Http\Controllers;

use App\Models\Omics;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class OmicsController extends Controller
{
    /**
     * 人体结构查询
     * @return \Illuminate\Http\JsonResponse
     */
    public function index(Request $request): \Illuminate\Http\JsonResponse
    {
//        DB::enableQueryLog();
        $res = Omics::join("mhcbinding", "mhcbinding.Sample ID", '=', "omics.Sample ID")
            ->join("blast", "blast.Reads ID", '=', "blast.Reads ID")
            ->where("omics.Source tissue", explode("_", $request->get("file"))[0])
            ->when(explode("_", $request->get("file"))[2]=='HLAI', function($query){
                $query->where("mhcbinding.HLA Allele", "like", "HLA%");
            })->when(explode("_", $request->get("file"))[2]=='HLAII', function($query){
                $query->where("mhcbinding.HLA Allele", "not like", "HLA%");
            })
            ->selectRaw(DB::raw("mhcbinding.*, omics.*,blast.*"))->paginate(10);
//        dd(DB::getQueryLog());
        return response()->json($res);
    }

    public function statistics(Request $request): \Illuminate\Http\JsonResponse
    {
//        DB::enableQueryLog();
        $res = Omics::join("mhcbinding", "mhcbinding.Sample ID", '=', "omics.Sample ID")
            ->groupBy("omics.Source tissue")
            ->selectRaw(DB::raw("omics.`Source tissue`,count(DISTINCT mhcbinding.`HLA Allele`) as HLACount,
            count(DISTINCT SUBSTRING_INDEX(SUBSTRING_INDEX(SUBSTRING_INDEX( `Reads ID`,'|', 2 ),'|',-1),'_',1)) as belongCount,
            count(DISTINCT SUBSTRING_INDEX(SUBSTRING_INDEX( `Reads ID`,'|', 2 ),'|',-1)) as categoryCount"))
            ->get();
//        dd(DB::getQueryLog());
        return response()->json($res);
    }
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Response
     */
    public function groupSourceTissue()
    {
//                DB::enableQueryLog();
        $group = Omics::groupBy("Source tissue")->select("Source tissue", DB::raw("count(1) as num"))->get();
//                dd(DB::getQueryLog());
        return response()->json($group);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        //
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        //
    }
}
